package be.iminds.ilabt.jfed.git;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.io.File;
import java.util.*;

public class GitFetcherCommandBuilder {
    private String gitUrl;
    private String repoSubDir;
    private String branch;
    private String username;
    private String password;
    private String privateKeyPem;
    private GitAuthPreferences.GitAuth gitAuth;
    private File targetDir;
    private boolean allowCreateDir;
    private boolean overwriteFilesInDir;

    public GitFetcherCommandBuilder() {
    }

    public static Optional<GitFetcherCommandBuilder> fromString(@Nonnull String command) {
        boolean isSpecialGit = command.trim().startsWith("git ");
        boolean isSpecialGithub = command.trim().startsWith("github ");

        if (!isSpecialGit && !isSpecialGithub)
            return Optional.empty();

        String[] gitSourceSpecSplit = command.trim().split(" ");
        if (gitSourceSpecSplit.length > 4)
            throw new IllegalArgumentException(
                    "'"+command+"' has too many parts. Syntax:" +
                            " git[hub] https://<githubTLD>/<org>/<repo>.git [<subdir> [<branch>]]");
        if (gitSourceSpecSplit.length < 2)
            throw new IllegalArgumentException(
                    "'"+command+"' has too few parts. Syntax:" +
                            " git[hub] https://<githubTLD>/<org>/<repo>.git [<subdir> [<branch>]]");

        assert gitSourceSpecSplit[0].trim().startsWith("git");
        String gitUrl = gitSourceSpecSplit[1];
        String subDir = gitSourceSpecSplit.length > 2 ? gitSourceSpecSplit[2] : null;
        String branch = gitSourceSpecSplit.length > 3 ? gitSourceSpecSplit[3] : null;

        if (subDir != null && !subDir.endsWith("/"))
            subDir += "/";

        return Optional.of(new GitFetcherCommandBuilder()
                .setGitUrl(gitUrl)
                .setRepoSubDir(subDir)
                .setBranch(branch)
                .setAllowCreateDir(false)
                .setOverwriteFilesInDir(false));
    }
    
    public GitFetcherCommandBuilder(@Nullable String gitUrl) {
        this.gitUrl = gitUrl;
    }

    public GitFetcherCommandBuilder setGitUrl(@Nullable String gitUrl) {
        this.gitUrl = gitUrl;
        return this;
    }

    public GitFetcherCommandBuilder setRepoSubDir(@Nullable String repoSubDir) {
        this.repoSubDir = repoSubDir;
        return this;
    }

    public GitFetcherCommandBuilder setBranch(@Nullable String branch) {
        this.branch = branch;
        return this;
    }

    public GitFetcherCommandBuilder setUsername(@Nullable String username) {
        if (username == null) {
            this.username = null;
            return this;
        }
        if (gitAuth != null || privateKeyPem != null)
            throw new IllegalStateException("cannot set username when other auth has been set");
        this.username = username;
        return this;
    }

    public GitFetcherCommandBuilder setPassword(@Nullable String password) {
        if (password == null) {
            this.password = null;
            return this;
        }
        if (gitAuth != null || privateKeyPem != null)
            throw new IllegalStateException("cannot set password when other auth has been set");
        this.password = password;
        return this;
    }

    public GitFetcherCommandBuilder setPrivateKeyPem(@Nullable String privateKeyPem) {
        if (privateKeyPem == null) {
            this.privateKeyPem = null;
            return this;
        }
        if (gitAuth != null || username != null || password != null)
            throw new IllegalStateException("cannot set privateKeyPem when other auth has been set");
        this.privateKeyPem = privateKeyPem;
        return this;
    }

    public GitFetcherCommandBuilder setGitAuth(@Nonnull GitAuthPreferences.GitAuth gitAuth) {
        if (gitAuth == null) {
            this.gitAuth = null;
            return this;
        }
        if (privateKeyPem != null || username != null || password != null)
            throw new IllegalStateException("cannot set gitAuth when other auth has been set");
        this.gitAuth = gitAuth;

        if (this.gitUrl == null)
            this.gitUrl = gitAuth.getRepoUrl();

        return this;
    }


    public GitFetcherCommandBuilder setTargetDir(File targetDir) {
        this.targetDir = targetDir;
        return this;
    }

    public GitFetcherCommandBuilder setAllowCreateDir(boolean allowCreateDir) {
        this.allowCreateDir = allowCreateDir;
        return this;
    }

    public GitFetcherCommandBuilder setOverwriteFilesInDir(boolean overwriteFilesInDir) {
        this.overwriteFilesInDir = overwriteFilesInDir;
        return this;
    }

    public GitFetcherCommand build() {
        if (gitUrl == null)
            throw new IllegalStateException("gitUrl is a required argument of GitFetchCommand");
        File usedTargetDir = targetDir;
        if (usedTargetDir == null) {
            //fall back to a tmp dir
            usedTargetDir = GitFetcherCommand.createTempDirectory();
        }

        if (gitAuth == null && username != null) {
            gitAuth = new GitAuthPreferences.GitAuthUserPass(gitUrl, username, password);
        }
        if (gitAuth == null && privateKeyPem != null) {
            gitAuth = new GitAuthPreferences.GitAuthSsh(gitUrl, privateKeyPem);
        }

        return new GitFetcherCommand(gitUrl, repoSubDir, branch,
                gitAuth,
                usedTargetDir, allowCreateDir, overwriteFilesInDir);
    }


    public String getGitUrl() { return gitUrl; }
    public String getRepoSubDir() { return repoSubDir; }
    public String getBranch() { return branch; }
}
